'''
Created on Mar 27, 2014

@author: Justin
'''

#This file stores all the entity templates.
#They are stored by name in a dict

import Graphics
import Physics
import Input
import ECS
import DeathSaveSystem
import TrapSystem
from Vector import Vec2d
from pyglet import resource
from pyglet import app
    
BLOCK_SIZE = 20

entityTypeList = []
entityTypeToTemplate = {}

def addEntityTemplate(entType, templateFunction):
    entityTypeList.append(entType)
    entityTypeToTemplate[entType] = templateFunction
    
def createEntity(entType, **kwargs):
    if entType in entityTypeList:
        ent = ECS.Entity(entType=entType)
        entityTypeToTemplate[entType](ent, **kwargs)
        return ent
    
def getTemplates():
    return entityTypeList

def loadLevel(levelName, systemManager):
    f = resource.file(levelName + ".dat", mode="r")
    fileLines = f.readlines()
    f.close()
    
    for line in fileLines:
        splitLine = line.split(";")
        argDict = eval(splitLine[1],{}, {})
        systemManager.addEntity(createEntity(splitLine[0], **argDict))

def saveLevel(levelName, systemManager):
    f = open(levelName + ".dat", mode="w")
    for ent in systemManager.entityList:
        if (not ent.persistant):
            pos = ent.getComponent("Position")
            argDict = {}
            argDict["x"] = pos.position.x
            argDict["y"] = pos.position.y
            
            f.write(ent.entType + ";" + str(argDict) + "\n")
    f.close()

def onPlay(systemManager):
    systemManager.clearEntities()
    systemManager.getEntityByName("Save").getComponent("SaveState").loadNow = True
    
def onRestart(systemManager):
    systemManager.clearEntities()
    loadLevel("level1", systemManager)
    systemManager.getEntityByName("Save").getComponent("SaveState").saveNow = True
    
def onExit(systemManager):
    app.exit()
    
def createPlayer(ent, x, y):
    right = Graphics.Renderable("StandRight.png")
    left = Graphics.Renderable("StandLeft.png")
    runright = Graphics.Renderable(imageName="RunRight.png", frames=4, framePeriod=1.0/8, width=28, height=40)
    runleft = Graphics.Renderable(imageName="RunLeft.png", frames=4, framePeriod=1.0/8, width=28, height=40)
    ent.addComponent(Graphics.Position(x,y)).addComponent(right).addComponent(Physics.PhysicsInfo(width=28, height=40, static=False, resolveCollision=True)).addComponent(Input.Controllable(runleft, left, runright, right)).addComponent(DeathSaveSystem.Killable("Death", True))
addEntityTemplate("Player", createPlayer)

def createPlayText(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.RenderText("Play", "Arial", 32, (255, 255, 255, 255), Graphics.FOREGROUND_LAYER)).addComponent(Input.MenuItem(0, onPlay, 2))
addEntityTemplate("PlayText", createPlayText)

def createRestartText(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.RenderText("Restart", "Arial", 32, (255, 255, 255, 255), Graphics.FOREGROUND_LAYER)).addComponent(Input.MenuItem(1, onRestart, 2))
addEntityTemplate("RestartText", createRestartText)

def createExitText(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.RenderText("Exit", "Arial", 32, (255, 255, 255, 255), Graphics.FOREGROUND_LAYER)).addComponent(Input.MenuItem(2, onExit, 2))
addEntityTemplate("ExitText", createExitText)    

def createDirtBlock(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("DirtBlock.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True))
addEntityTemplate("DirtBlock", createDirtBlock)

def createGrassBlock(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("GrassBlock.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True))
addEntityTemplate("GrassBlock", createGrassBlock)

def createUsedBlock(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("UsedBlock.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True))
addEntityTemplate("UsedBlock", createUsedBlock)

def createGreyBlock(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("GreyBlock.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True))
addEntityTemplate("GreyBlock", createGreyBlock)

def createMetalBlock(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("MetalBlock.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True))
addEntityTemplate("MetalBlock", createMetalBlock)

def createApple(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Apple.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("Apple", createApple)

def createAppleUp(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Apple.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("AppleUp", createAppleUp)

def createAppleDown(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Apple.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x+3, y-1000, 14, 2000, Vec2d(0, -600), "Player"))
addEntityTemplate("AppleDown", createAppleDown)

def createTree(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Tree.png"))
addEntityTemplate("Tree", createTree)

def createDeath(ent, x, y):
    ent.addComponent(Graphics.Position(x, y)).addComponent(Graphics.Renderable(imageName="Death.png", frames=4, framePeriod=1.0/4, width=28, height=40))
addEntityTemplate("Death", createDeath)

def createSign(ent, x, y, level, save):
    ent.addComponent(Graphics.Position(x, y)).addComponent(Graphics.Renderable(imageName="Sign.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.LevelTeleporter(level, "Player", save))
addEntityTemplate("Sign", createSign)

def createLadder(ent, x, y, level, save):
    ent.addComponent(Graphics.Position(x, y)).addComponent(Graphics.Renderable(imageName="Ladder.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.LevelTeleporter(level, "Player", save))
addEntityTemplate("Ladder", createLadder)

def createLadder2(ent, x, y, level, save):
    ent.addComponent(Graphics.Position(x, y)).addComponent(Graphics.Renderable(imageName="Ladder2.png")).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.LevelTeleporter(level, "Player", save))
addEntityTemplate("Ladder2", createLadder2)

def createSpikeDown(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeDown.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("SpikeDown", createSpikeDown)

def createSpikeUp(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeUp.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("SpikeUp", createSpikeUp)

def createSpikeLeft(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeLeft.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("SpikeLeft", createSpikeLeft)

def createSpikeRight(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeRight.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("SpikeRight", createSpikeRight)

def createSpikeFlyUp(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeUp.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x+3, y, 14, 1000, Vec2d(0, 600), "Player"))
addEntityTemplate("SpikeFlyUp", createSpikeFlyUp)

def createSpikeFlyDown(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeDown.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x+3, y-1000, 14, 2000, Vec2d(0, -600), "Player"))
addEntityTemplate("SpikeFlyDown", createSpikeFlyDown)

def createSpikeFlyRight(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeRight.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x-3, y-3, 1000, 14, Vec2d(600, 0), "Player"))
addEntityTemplate("SpikeFlyRight", createSpikeFlyRight)

def createSpikeUpLeft(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("SpikeUp.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x-1000, y-3, 1000, 14, Vec2d(-600, 0), "Player"))
addEntityTemplate("SpikeUpLeft", createSpikeUpLeft)

def createAppleRight(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Apple.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE)).addComponent(DeathSaveSystem.Dangerous(True))
    ent.addComponent(TrapSystem.BoobyTrap(x-3, y-3, 1000, 14, Vec2d(600, 0), "Player"))
addEntityTemplate("AppleRight", createAppleRight)

def createLava(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("Lava.png", frames=4, framePeriod=1.0/4, width=20, height=20, layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(0,0), acceleration=Vec2d(0,500), width=BLOCK_SIZE,height=BLOCK_SIZE, static=True)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("Lava", createLava)

def createCageYell(ent, x, y):
    ent.addComponent(Graphics.Position(x,y)).addComponent(Graphics.Renderable("CageYell.png", layer=Graphics.FOREGROUND_LAYER)).addComponent(Physics.PhysicsInfo(velocity=Vec2d(-30,0), acceleration=Vec2d(0,500), width=335,height=480)).addComponent(DeathSaveSystem.Dangerous(True))
addEntityTemplate("CageYell", createCageYell)

def createBackGround(ent, imageName):
    ent.addComponent(Graphics.Position(0, 0)).addComponent(Graphics.Renderable(imageName + ".png", layer=Graphics.BACKGROUND_LAYER))
addEntityTemplate("BackGround", createBackGround)