'''
Created on Mar 20, 2014

@author: Justin
'''

from pyglet import app, clock, resource
from pyglet.window import Window
from pyglet.app import windows

import ECS
import Graphics
import Physics
from Vector import Vec2d
from Input import KeyMouseState
import Input
import EntityCreator
import DeathSaveSystem
import TrapSystem

#Need custom game loop since pyglet is variable timestep
#and a fixed timestep is needed for stability
class MainEventLoop(app.EventLoop):
    
    def idle(self):
        dt = clock.tick(True)

        # Redraw all windows
        for window in windows:
            if window.invalid:
                window.switch_to()
                window.update(dt)
                window.dispatch_event('on_draw')
                window.flip()

        # Update timout
        return clock.get_sleep_time(False)

class MainWindow(Window):
    '''
    This is the main game window. It runs the Entity-Component system and
    has to take special care for the graphics system since it can only be
    run at certain times.
    '''

    def __init__(self, width=640, height=480, windowCaption=""):
        '''
        Start up all the systems and load the menu.
        '''
        super(MainWindow, self).__init__(width, height, windowCaption)
        self.MAX_LAG_TIME = 1.0 / 5
        self.LOGIC_UPDATE_PERIOD = 1.0 / 30
        self.timeAccumulator = 0.0
        self.alpha = 0.0
        self.systemManager = ECS.SystemManager()
        self.graphicsSystem = Graphics.GraphicsSystem(self.systemManager)
        self.systemManager.addSystem(self.graphicsSystem)
        self.keyState = KeyMouseState()
        self.push_handlers(self.keyState)
        self.systemManager.addSystem(Input.InputSystem(self.systemManager, self.keyState))
        self.systemManager.addSystem(Physics.PhysicsSystem(self.systemManager,gravity=Vec2d(0,-500)))
        self.systemManager.addSystem(DeathSaveSystem.DeathSaveSystem(self.systemManager))
        self.systemManager.addSystem(TrapSystem.TrapSystem(self.systemManager))
        
        saveEnt = ECS.Entity(name="Save",persistant=True).addComponent(DeathSaveSystem.SaveState(loadNow=False))
        self.systemManager.addEntity(saveEnt)
        EntityCreator.loadLevel("menu", self.systemManager)
        
    def on_draw(self):
        self.clear()
        self.graphicsSystem.update(self.alpha)
    
    #Update calls every system's update function at a fixed timestep to
    #increase stability
    def update(self, timeElapsed):
        if (timeElapsed > self.MAX_LAG_TIME):
            timeElapsed = self.MAX_LAG_TIME
            print "bad"
        
        self.timeAccumulator += timeElapsed
        
        while (self.timeAccumulator >= self.LOGIC_UPDATE_PERIOD):
            self.systemManager.update(self.LOGIC_UPDATE_PERIOD)
            
            self.keyState.nextFrame()
            
            self.timeAccumulator -= self.LOGIC_UPDATE_PERIOD
            
        self.alpha = self.timeAccumulator / self.LOGIC_UPDATE_PERIOD
    
if __name__ == '__main__':
    resource.path = [".", "images", "levels"] #Set up places to look for files
    resource.reindex()
    window = MainWindow()
    clock.set_fps_limit(60)
    MainEventLoop().run()